package require Tcl 8.4
package require Itcl
package require iblowfish

#
# ----------------------------------------------------------------------
# This file contains the public API for the pwsafe package
# ----------------------------------------------------------------------
#

namespace eval pwsafe {}

#
# ----------------------------------------------------------------------
# createFromStream: create a pwsafe::db object from a stream object
# ----------------------------------------------------------------------
#

proc pwsafe::createFromStream {stream password {percentvar ""}} {
    if {$percentvar != ""} {
	upvar $percentvar pcv
	set pcvp "pcv"
    } else {
	set pcvp ""
    }

    set db [namespace current]::[pwsafe::db #auto $password]
    set reader [namespace current]::[pwsafe::io::reader #auto $db $stream]

    if {[catch {$reader readFile $pcvp} oops]} {
	set origErrorInfo $::errorInfo
	itcl::delete object $reader
	itcl::delete object $db
	error $oops $origErrorInfo
    }

    itcl::delete object $reader
    return $db
}

#
# ----------------------------------------------------------------------
# createFromFile: create a pwsafe object from a file
# ----------------------------------------------------------------------
#

proc pwsafe::createFromFile {fileName password {percentvar ""}} {
    if {$percentvar != ""} {
	upvar $percentvar pcv
	set pcvp "pcv"
    } else {
	set pcvp ""
    }

    if {[catch {set size [file size $fileName]}]} {
	set size -1
    }

    set file [open $fileName "r"]
    fconfigure $file -translation binary
    set stream [namespace current]::[pwsafe::io::streamreader #auto $file $size]

    if {[catch {set db [pwsafe::createFromStream $stream $password $pcvp]} oops]} {
	set origErrorInfo $::errorInfo
	itcl::delete object $stream
	catch {close $file}
	error $oops $origErrorInfo
    }

    itcl::delete object $stream

    if {[catch {close $file} oops]} {
	itcl::delete object $db
	error $oops
    }

    return $db
}

#
# ----------------------------------------------------------------------
# createFromString: create a pwsafe object from an (in-memory) string
# ----------------------------------------------------------------------
#

proc pwsafe::createFromString {data password {percentvar ""}} {
    if {$percentvar != ""} {
	upvar $percentvar pcv
	set pcvp "pcv"
    } else {
	set pcvp ""
    }

    set stream [namespace current]::[pwsafe::io::stringreader #auto $data]

    if {[catch {set db [pwsafe::createFromStream $stream $password $pcvp]} oops]} {
	set origErrorInfo $::errorInfo
	itcl::delete object $stream
	error $oops $origErrorInfo
    }

    itcl::delete object $stream
    return $db
}

#
# ----------------------------------------------------------------------
# writeToFile: write a pwsafe object to a file
# ----------------------------------------------------------------------
#

proc pwsafe::writeToFile {db fileName {percentvar ""}} {
    if {$percentvar != ""} {
	upvar $percentvar pcv
	set pcvp "pcv"
    } else {
	set pcvp ""
    }

    #
    # Write to a temporary file first, then make sure that the
    # real destination file does not exist (delete if it does),
    # then rename the file. This way, the existing database is
    # not lost, if something goes wrong.
    #

    set tmpFileName $fileName
    append tmpFileName ".tmp"

    set file [open $tmpFileName "w"]
    fconfigure $file -translation binary

    set stream [namespace current]::[pwsafe::io::streamwriter #auto $file]
    set writer [namespace current]::[pwsafe::io::writer #auto $db $stream]

    if {[catch {$writer writeFile $pcvp} oops]} {
	set origErrorInfo $::errorInfo
	itcl::delete object $writer
	itcl::delete object $stream
	catch {close $file}
	catch {file delete $tmpFileName}
	error $oops $origErrorInfo
    }

    itcl::delete object $writer
    itcl::delete object $stream
    close $file

    #
    # Done writing to temporary file.
    #

    if {[file exists $fileName]} {
	file delete -- $fileName
    }

    file rename -- $tmpFileName $fileName
}

#
# ----------------------------------------------------------------------
# writeToString: write a pwsafe object to a string
# ----------------------------------------------------------------------
#

proc pwsafe::writeToString {db {percentvar ""}} {
    if {$percentvar != ""} {
	upvar $percentvar pcv
	set pcvp "pcv"
    } else {
	set pcvp ""
    }


    set stream [namespace current]::[pwsafe::io::stringwriter #auto]
    set writer [namespace current]::[pwsafe::io::writer #auto $db $stream]

    if {[catch {$writer writeFile $pcvp} oops]} {
	set origErrorInfo $::errorInfo
	itcl::delete object $writer
	itcl::delete object $stream
	catch {close $file}
	error $oops $origErrorInfo
    }

    set result [$stream cget -data]
    itcl::delete object $writer
    itcl::delete object $stream
    return $result
}

#
# ----------------------------------------------------------------------
# pwsafe::dumpAllRecords: print all records in a human readable manner
# ----------------------------------------------------------------------
#

proc pwsafe::dumpAllRecords {db out} {
    foreach rn [$db getAllRecordNumbers] {
	pwsafe::io::dumpRecord $db $out $rn
    }
}

#
# ----------------------------------------------------------------------
# Source the pwsafe::db implementation and the various helpers
# ----------------------------------------------------------------------
#

set pwsafeDir [file dirname [info script]]
source [file join $pwsafeDir "pwsafe-int.tcl"]
source [file join $pwsafeDir "pwsafe-db.tcl"]
source [file join $pwsafeDir "pwsafe-io.tcl"]

#
# ----------------------------------------------------------------------
# finishing touches
# ----------------------------------------------------------------------
#

package provide pwsafe 0.1
